import {Project, SyntaxKind, Node} from "ts-morph";
import prettier from "prettier/standalone";
import parserTypescript from "prettier/plugins/typescript";
import estree from "prettier/plugins/estree";

interface CompilerHooks {
  onload?: (compiler: Compiler) => Promise<any>;
  onInserted?: (data: any) => Promise<any>;
  onRemoved?: (data: any) => Promise<any>;
  onPrettier?: (data: string) => Promise<any>;
  onError?: (e: Error) => Promise<any>;
}
interface CompilerOptions {
  sourceCode: string;
  fileNameOrPath: string;
  hooks: CompilerHooks;
}

export enum CreateProperty {
  'State' = 'State',
  'Function' = 'Function',
}

export type CreatePropertyType = 'string' | 'number' | 'object' | 'boolean' | 'function' |'array'

export interface CreatePropertyItem {
  type: CreatePropertyType;
  name: string;
  comment?: string;
  value?: any;
  params?: string[],
}

export interface CreatePropertyConfig {
  override?: boolean;
}

interface ParseFunctionResult {
  name: string;
  params: string[];
  body: string
}
class Compiler {

  private readonly options: CompilerOptions;

  private readonly fileNameOrPath = 'LowcodeComponent.tsx'

  private project!: Project
  constructor(options: CompilerOptions) {
    this.options = options;
    this.fileNameOrPath = options.fileNameOrPath || 'LowcodeComponent.tsx' as any
    this.init()
  }

  /**
   * 初始化
   * @private
   */
  private init() {
    try {
      this.project = new Project({
        useInMemoryFileSystem: true
      });
      void this.options?.hooks?.onload?.(this);
    }catch (e) {
      void this.options?.hooks?.onError?.(e as any);
    }
  }

  private getSourceFile() {
    if (!this.project) {
      this.project = new Project({
        useInMemoryFileSystem: true
      })
    }
    let file = this.project.getSourceFile(this.fileNameOrPath);
    if (!file) {
      file = this.project.createSourceFile("LowcodeComponent.tsx", this.options.sourceCode);
    }
    return file;
  }

  /**
   * 获取默认值
   * @param variableType
   */
  public getDefaultValue(variableType: CreatePropertyType) {
    switch (variableType.toLowerCase()) {
      case "string":
        return `''`;
      case "number":
        return `0`;
      case "bool":
      case "boolean":
        return `false`;
      case "object":
        return `{}`;
      case "function":
        return ``;
      case "array":
        return `[]`;
      default:
        return `null`;
    }
  }

  /**
   * 读取当前的code
   */
  get code() {
    const file = this.getSourceFile();
    return file.getFullText();
  }

  public async getPrettierCode() {
    const code = this.code;
    return await this.prettierCode(code);
  }

  /**
   * 删除属性及相关注释
   * @param stateObjectLiteral
   * @param property
   * @private
   */
  private removePropertyWithComments(stateObjectLiteral: any, property: any) {
    const propertyStart = property.getStart();
    const sourceFile = stateObjectLiteral.getSourceFile();

    // 遍历所有注释范围
    const commentRanges = sourceFile.getDescendantsOfKind(SyntaxKind.SingleLineCommentTrivia);
    for (const comment of commentRanges) {
      if (comment.getStart() < propertyStart) {
        comment.remove(); // 删除目标注释
      }
    }
    property.remove();
  }


  /**
   * 创建属性
   * @type<CreateProperty> 类型
   * @param type
   * @param item
   * @param config
   */
  public createProperty(type: CreateProperty | string, item: CreatePropertyItem, config?: CreatePropertyConfig): Promise<{type: CreateProperty | string; item:CreatePropertyItem; code: string }> {
    return new Promise(async (resolve, reject) => {
      try {
        const { value, name, params = [], comment, type: itemType} = item;
        const file = this.getSourceFile();
        const classDeclaration = file.getClass(() => true);
        if (!classDeclaration) {
          throw new Error("classDeclaration property not found!");
        }

        if (type === CreateProperty.State) {
          const stateProperty = file.getClass("LowcodeComponent")?.getProperty?.("state");
          if (!stateProperty) {
            throw new Error("State property not found!");
          }
          const stateObjectLiteral = stateProperty.getInitializerIfKindOrThrow(
            SyntaxKind.ObjectLiteralExpression
          );
          if (config?.override) {
            const existingProperty = stateObjectLiteral.getProperty(name);
            if (existingProperty) {
              this.removePropertyWithComments(stateObjectLiteral, existingProperty);
              // existingProperty.remove(); // 删除已存在的属性
            }
          }
          stateObjectLiteral.addPropertyAssignment({
            initializer: value || this.getDefaultValue(itemType),
            name: name,
            leadingTrivia: comment ? `// ${comment}\n` : undefined, // 添加注释
          });
        }
        if (type === CreateProperty.Function) {
          if (config?.override) {
            // 如果已经存在，则直接移除掉
            const existingMethod = classDeclaration.getMethod(name);
            if (existingMethod) {
              existingMethod.remove(); // 删除已存在的方法
            }
          }
          classDeclaration.addMethod({
            name: name,
            parameters: params.map((param) => ({ name: param })),
            statements: value || this.getDefaultValue(itemType),
            docs: comment? [`\n ${comment} `]: undefined,
          });
        }
        const code = file.getFullText();
        const newCode = await this.prettierCode(code);
        await this.options?.hooks?.onInserted?.(newCode);
        resolve({
          type,
          code: newCode,
          item
        });
      }catch (e) {
        await this.options?.hooks?.onError?.(e as any);
        reject(e);
      }
    })
  }


  /**
   * 移除掉属性
   * @param type
   * @param name
   */
  public deleteProperty(type: CreateProperty | string, name: string): Promise<boolean> {
    return new Promise(async (resolve, reject) => {
      try {
        const file = this.getSourceFile();
        const classDeclaration = file.getClass(() => true);
        if (!classDeclaration) {
          throw new Error("classDeclaration property not found!");
        }
        if (type === CreateProperty.State) {
          const stateProperty = file.getClass("LowcodeComponent")?.getProperty?.("state");
          if (!stateProperty) {
            throw new Error("State property not found!");
          }
          const stateObjectLiteral = stateProperty.getInitializerIfKindOrThrow(
            SyntaxKind.ObjectLiteralExpression
          );
          const existingProperty = stateObjectLiteral.getProperty(name);
          if (existingProperty) {
            this.removePropertyWithComments(stateObjectLiteral, existingProperty);
            await this.options?.hooks?.onRemoved?.(true);
          }
        }
        if (type === CreateProperty.Function) {
          // 如果已经存在，则直接移除掉
          const existingMethod = classDeclaration.getMethod(name);
          if (existingMethod) {
            existingMethod.remove(); // 删除已存在的方法
            await this.options?.hooks?.onRemoved?.(true);
          }
          //classDeclaration.removeName()
        }
        resolve(true);
      }catch (e) {
        await this.options?.hooks?.onError?.(e as any);
        reject(e);
      }
    })

  }

  /**
   * 提取节点的 JSDoc 注释
   * @param node ts-morph 节点
   * @returns 注释字符串或 undefined
   */
  getJsDocComment(node: Node): string | undefined | any {
    // 获取前置块注释或 JSDoc
    const leadingComments = node.getLeadingCommentRanges();
    const jsDocs = leadingComments
      .filter(comment => comment.getKind() === SyntaxKind.MultiLineCommentTrivia)
      .map(comment =>
        comment
          .getText()
          .replace(/^\/\*\*?/, "") // 去掉起始的 /** 或 /*
          .replace(/\*\/$/, "")   // 去掉结尾的 */
          .split("\n")            // 按行分割
          .map(line => line.replace(/^\s*\*?/, "").trim()) // 移除每行开头的 * 和多余空格
          .join("\n")             // 重新合并为字符串
          .trim()
      );

    // 获取尾部行内注释
    const trailingComments = node.getTrailingCommentRanges();
    const inlineComments = trailingComments
      .filter(comment => comment.getKind() === SyntaxKind.SingleLineCommentTrivia)
      .map(comment => comment.getText().replace(/^\/\//, "").trim());

    // 合并所有注释
    const allComments = [...jsDocs, ...inlineComments].filter(Boolean);

    return allComments.length > 0 ? allComments.join("\n") : undefined;
  }

  private getValueType(property: any) {
    // 属性值和类型
    const initializer = property.getInitializer();
    let type = "string";

    if (initializer) {
      const kind = initializer.getKind();

      switch (kind) {
        case SyntaxKind.StringLiteral:
          type = "string";
          break;

        case SyntaxKind.NumericLiteral:
          type = "number";
          break;

        case SyntaxKind.TrueKeyword:
        case SyntaxKind.FalseKeyword:
          type = "boolean";
          break;

        case SyntaxKind.NullKeyword:
          type = "null";
          break;

        case SyntaxKind.UndefinedKeyword:
          type = "undefined";
          break;

        case SyntaxKind.ArrayLiteralExpression:
          type = "array";
          break;

        case SyntaxKind.ObjectLiteralExpression:
          type = "object";
          break;

        case SyntaxKind.FunctionExpression:
        case SyntaxKind.ArrowFunction:
          type = "function";
          break;

        default:
          type = "string";
          break;
      }
    }

    return type;
  }

  /**
   * 获取属性
   * @param type
   * @param name
   */
  public getProperty(type: CreateProperty | string, name: string): Promise<CreatePropertyItem> {
    return new Promise(async (resolve, reject) => {
      try {
        const file = this.getSourceFile();
        const classDeclaration = file.getClass(() => true);
        if (!classDeclaration) {
          throw new Error("Class declaration not found!");
        }
        if(type === CreateProperty.State) {
          // 获取 "state" 属性
          const stateProperty = file.getClass("LowcodeComponent")?.getProperty?.("state");
          if (!stateProperty) {
            throw new Error("State property not found!");
          }

          // 确保 "state" 是对象初始化器
          const stateObjectLiteral = stateProperty.getInitializerIfKindOrThrow(
            SyntaxKind.ObjectLiteralExpression
          );

          // 查找指定属性
          const existingProperty = stateObjectLiteral.getProperty(name);
          if (!existingProperty || !Node.isPropertyAssignment(existingProperty)) {
            reject(null);
            return;
          }

          const getJsDocComment = (stateObjectLiteral: any, property: any) => {
            const propertyStart = property.getStart();
            const sourceFile = stateObjectLiteral.getSourceFile();
            const list = []

            // 遍历所有注释范围
            const commentRanges = sourceFile.getDescendantsOfKind(SyntaxKind.SingleLineCommentTrivia);
            for (const comment of commentRanges) {
              if (comment.getStart() < propertyStart) {
                const text = comment.getText(true)
                if (text.includes('//')) {
                  const str = text.split('//');
                  if (str.length > 1) {
                    list.push(text.replace('//', '').trim())
                  }
                }

              }
            }
            return list.join('\n');
          }

          const valueType = this.getValueType(existingProperty)

          // 提取属性名称
          const propertyName = existingProperty.getName();

          // 提取属性值
          const propertyValue = existingProperty.getInitializer()?.getText() || "";
          let value = propertyValue?  await this.prettierCode(propertyValue): '';
          if (value.includes(';')) {
            value = value.replace(';', '')
          }

          // 提取注释
          const propertyComment = getJsDocComment(stateObjectLiteral, existingProperty);

          resolve({
            name: propertyName,
            comment: propertyComment,
            value: value,
            type: valueType as any
          });
        }
        if (type === CreateProperty.Function) {
          // 获取指定函数
          const functionDeclaration = classDeclaration.getMethod(name);

          if (!functionDeclaration) {
            return reject(new Error('not found'))
          }

          // 提取函数信息
          const name1 = functionDeclaration.getName() || "";
          const params = functionDeclaration.getParameters().map(param => param.getName());
          //const body = functionDeclaration.getBodyText() || "";
          const value = functionDeclaration.getText() || "";
          const comment = this.getJsDocComment(functionDeclaration)
          const newCode = await this.prettierCode(`function ${value}`)

          resolve({
            name: name1,
            comment: comment,
            value: newCode,
            type: 'function',
            params
          });

        }


      } catch (e) {
        await this.options?.hooks?.onError?.(e as any);
        reject(e);
      }
    })

  }

  /**
   * 格式化代码
   * @param sourceCode
   */
  public async prettierCode(sourceCode: string): Promise<string> {
    const result = await prettier.format(sourceCode, {
      parser: "typescript", // 选择解析器
      plugins: [estree, parserTypescript], // 加载插件
      singleQuote: true, // 其他格式化规则
      trailingComma: "all",
    });
    void this.options.hooks?.onPrettier?.(result)
    return result
  }

  /**
   * 解析函数的名称、参数集合和函数体
   * @param code 需要解析的代码字符串
   * @param functionName 需要提取的函数名称
   * @returns 包含函数名称、参数集合和函数体的对象
   */
  public async parseFunction(code: string, functionName: string): Promise<ParseFunctionResult> {
    return new Promise((resolve, reject) => {
      const project = new Project({
        useInMemoryFileSystem: true
      });
      try {
        // 创建临时文件并添加代码
        const sourceFile = project.createSourceFile("temp.ts", code);

        // 获取指定函数
        const functionDeclaration = sourceFile.getFunction(functionName);

        if (!functionDeclaration) {
          return reject(new Error('not found'))
        }

        // 提取函数信息
        const name = functionDeclaration.getName() || "";
        const params = functionDeclaration.getParameters().map(param => param.getName());
        const body = functionDeclaration.getBodyText() || "";

        resolve({
          name,
          params,
          body
        })
      } catch (e) {
        reject(e)
      }
    })
  }

  /**
   * 编译器销毁
   */
  public dispose() {

  }
}

export {
  Compiler
}
